// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysslbase.h 717 2010-08-02 09:44:58Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** Base functionality for OpenSSL integration.
**
*****************************************************************************/

#ifndef FANCYSSL_BASE_H
#define FANCYSSL_BASE_H

#ifdef WIN32
# ifdef __GNUC__
#  ifdef BUILD_fancyssl2_DLL
#   define FANCYSSL_API __attribute__((dllexport))
#  else
#   define FANCYSSL_API __attribute__((dllimport))
#  endif
# else
#  ifdef BUILD_fancyssl2_DLL
#   define FANCYSSL_API __declspec(dllexport)
#  else
#   define FANCYSSL_API __declspec(dllimport)
#  endif
# endif
#else
# ifdef __GNUC__
#  define FANCYSSL_INTERNAL_API __attribute__((visibility("hidden")))
# endif
#endif

#ifndef FANCYSSL_INTERNAL_API
#define FANCYSSL_INTERNAL_API
#endif

#ifndef FANCYSSL_API
#define FANCYSSL_API
#endif

#include <fancyobject.h>

namespace Fancy {
  namespace SSL {

  /**
   * This is the base calss for all objects, which encapsulate an OpenSSL
   * handle. The constructor of this class ensures, that the openssl library is
   * correctly initialized.
   */
  class FANCYSSL_API Base : virtual public Fancy::Object
  {
  public:
	  typedef Fancy::ObjPtr<Base> ptr_t;

	  /**
	   * Perform one-time initializations, if not done so far.
	   * This is needed for static functionality, which is executed before
	   * actually calling the Base ctor. This method my be called as often as you
	   * want and from concurrent threads. It is guaranteed, that the initializations
	   * are only performed once.
	   */
	  static void init();

	  /**
	   * Return the last formatted openssl error message which occurred inside this thread.
	   */
	  static std::string errorMessage();

  protected:

	  /**
	   * Perform one-time initializations by calling @c Base::init()
	   */
	  Base();
	  virtual ~Base();
  };
  }
}

#endif
