/*
 * fancysslchannelbio.h
 *
 *  Created on: 28.06.2009
 *      Author: wglas
 */

#ifndef FANCYSSLCHANNELBIO_H_
#define FANCYSSLCHANNELBIO_H_

#include <fancysslbase.h>

// fwd decls from openssl.
extern "C" {
	struct bio_st;
	typedef struct bio_st BIO;
};

namespace Fancy {

class Channel;

namespace SSL {

/**
 * An adapter for Fancy::Channel to an OpenSSL BIO.
 */
class FANCYSSL_API ChannelBIO: public Fancy::SSL::Base {

	BIO *bio;

protected:
	ChannelBIO(BIO *_bio);
	virtual ~ChannelBIO();

public:

	typedef Fancy::ObjPtr<ChannelBIO> ptr_t;

	/**
	 * @return a BIO, which references the given channel. The caller is responsible
	 * for calling @c BIO_free() on the returned pointer, which will dereference
	 * the given channel.
	 */
	static BIO *newBIOInstance(const Fancy::ObjPtr<Fancy::Channel> &channel);

	/**
	 * Return a new @c ChannelBIO object, which manages a BIO referencing the
	 * given channel.
	 */
	static ptr_t newInstance(const Fancy::ObjPtr<Fancy::Channel> &channel);

	/**
	 * Return the BIO managed by this @c ChannelBIO instance. The caller
	 * must not call @c BIO_free on the returned pointer.
	 */
	BIO *getBIO() const;
};

}

}

#endif /* FANCYSSLCHANNELBIO_H_ */
