// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancyssldigest.h 311 2009-06-29 20:24:34Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A class for creating message digests.
**
*****************************************************************************/

#ifndef FANCYSSL_DIGEST_H
#define FANCYSSL_DIGEST_H

#include <fancysslbase.h>
#include <fancyarray.h>

// fwd decls from openssl.
extern "C" {
  struct env_md_ctx_st;
  typedef struct env_md_ctx_st EVP_MD_CTX;
};

namespace Fancy {

  class Channel;

  namespace SSL {

  /**
   *  Handles an OpenSSL BIO Structure for creating a message digest.
  */
  class FANCYSSL_API Digest : public Base
  {

  protected:
	  /**
	   * Construct a digest object taking ownership of a fully initialized
	   * digest context.
	   */
	  Digest (EVP_MD_CTX *_ctx);

	  /**
	   * Free the associated OpenSSL context.
	   */
	  ~Digest();

  public:

	  typedef Fancy::ObjPtr<Digest> ptr_t;

	  /**
	   * @return a new instancee for the given algorithm name.
	   */
      ptr_t newInstance(const char *algo);

      /**
       * Update the digest with the given binary data.
       */
      void addMessage(const uint8_t *m, size_t len);

      /**
       * Update the digest with all remaining data from the given I/O channel.
       */
      void addChannel(const Fancy::ObjPtr<Fancy::Channel> &channel);

      /**
       * Update the digest with all data from the given file.
       */
      void addFile(const char *filename);

      /**
       * return the value of the digest as an upper case hexadecimal string.
       */
      std::string getHexDigest() const;

      /**
       * Return the value of the digest as a byte array.
       */
      Fancy::ArrayObject<uint8_t>::ptr_t getDigest() const;

      /**
       * Return the result of @c getHexDigest().
       */
      std::string toString() const;

    private:
      Digest (const Digest &a);
      Digest &operator= (const Digest &a);

      EVP_MD_CTX *ctx;
    };
  }
}

#endif
