// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysslx509name.h 311 2009-06-29 20:24:34Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A class for handling OpenSSL X509 DNs.
**
*****************************************************************************/

#ifndef FANCYSSL_X509NAME_H
#define FANCYSSL_X509NAME_H

#include <fancysslbase.h>

// fwd decls from openssl.
extern "C" {
  struct X509_name_st;
  typedef struct X509_name_st X509_NAME;
};

namespace Fancy {
  namespace SSL {
    class FANCYSSL_API X509Name : public Base
    {
    protected:
        X509Name ();
        X509Name (X509_NAME *_x509_name);

        ~X509Name();

    public:
    	typedef Fancy::ObjPtr<X509Name> ptr_t;

    	/**
    	 * Return a pointer to a new, empty instance of an @c X509Name.
    	 */
    	static ptr_t newInstance();

    	/**
    	 * Parse the given rfc2253 string and return a new instance.
    	 */
    	static ptr_t newInstance(const std::string &rfc2253);

    	/**
    	 * Aquires the given OpenSSL handle and automagically free the handle,
    	 * if the object goes out of scope. The caller must not call \c X509_free
    	 * on the supplied resource.
    	 */
    	static ptr_t newInstance(X509_NAME *x509_name);

    	/**
    	 * return a cloned instance.
    	 */
    	ptr_t clone() const;

    	/**
    	 * Return the RFC2253 representation of this @c X509Name.
    	 */
    	std::string toString() const;

    	bool lessThan(const Fancy::Object &o) const;
    	bool equals(const Fancy::Object &o) const;

        /**
         * @return The OpenSSL handle of this object. THe caller must not call
         *         @c X509_NAME_free on the returned handle.
    	 */
    	X509_NAME *getX509_NAME () const;

        /**
         * @return A duplicate of the OpenSSL handle of this object.
         * The caller is responsible for calling @c X509_NAME_free
         * on the returned handle.
    	 */
    	X509_NAME *dup () const;

    	/**
    	 * Adds the an utf8 encoded name entry corresponding to the given component
    	 * name to this object.
    	 */
    	void addDNcomponent(const char *comp, const unsigned char *utf8);

        /**
           Gets the utf8 encoded name entry corresponding to the given component name.
        */
    	std::string getDNComponentUtf8(const char *comp) const;

        /**
          Gets a oneline, user-readable description of the subject's distinguished name.
          This is OpenSSL-specific and returns a format, which may not be interpreted by
          other software tools.

          For a standardized format use @c getRFC2253().
        */
    	std::string getOneLine() const;

    	/**
    	 * Gets a utf8 oneline description of the subject's distinguished name
         * conforming to RFC2253.
    	 */
    	std::string getRFC2253() const;

    private:
    	X509_NAME *x509_name;
    };
  }
}

#endif
